import logging
import os
import sys
from typing import Dict

import structlog

# configured for JSON logging
# see: http://www.structlog.org/en/stable/standard-library.html#rendering-within-structlog

structlog.configure(
    processors=[
        # This performs the initial filtering, so we don't
        # evaluate e.g. DEBUG when unnecessary
        structlog.stdlib.filter_by_level,
        # Adds logger=module_name (e.g __main__)
        structlog.stdlib.add_logger_name,
        # Adds level=info, debug, etc.
        structlog.stdlib.add_log_level,
        # Performs the % string interpolation as expected
        structlog.stdlib.PositionalArgumentsFormatter(),
        # add ISO9601 stamp: e.g. 2019-11-06T19:53:41.189600Z
        structlog.processors.TimeStamper(fmt="iso"),
        # Include the stack when stack_info=True
        structlog.processors.StackInfoRenderer(),
        # Include the exception when exc_info=True
        # e.g log.exception() or log.warning(exc_info=True)'s behavior
        structlog.processors.format_exc_info,
        # Decodes the unicode values in any kv pairs
        structlog.processors.UnicodeDecoder(),
        structlog.processors.JSONRenderer(sort_keys=True)
    ],
    # Our "event_dict" is explicitly a dict
    # There's also structlog.threadlocal.wrap_dict(dict) in some examples
    # which keeps global context as well as thread locals
    context_class=dict,
    # Provides the logging.Logger for the underlaying log call
    logger_factory=structlog.stdlib.LoggerFactory(),
    # Provides predefined methods - log.debug(), log.info(), etc.
    wrapper_class=structlog.stdlib.BoundLogger,
    # Caching of our logger
    cache_logger_on_first_use=True,
)


class LogUtil:

    @staticmethod
    def init_logger(env_var_name: str = 'LOG_LEVEL',
                    default_level: str = 'INFO',
                    correlation_id_key_val: Dict[str, str] = None) -> logging.Logger:
        """

        :param correlation_id_key_val: [optional] correlation {key: val} to appear in all log statements.
        :type correlation_id_key_val: str
        :param env_var_name: [optional] Looks for log level [DEBUG|INFO|...] set at this ENV var name
        :type env_var_name: str
        :param default_level: [optional, defaults to INFO] Run this level if env_var_name not found
        :type default_level: str
        :return:
        :rtype: Logging.Logger
        """
        log_level = os.getenv(env_var_name, default_level)
        logging.basicConfig(
            format="%(message)s",
            stream=sys.stdout,
            level=log_level.upper(),
        )
        log = structlog.getLogger()
        # gotta be a better way to do this??
        if os.getenv('TESTING_RUN', False):
            fh = logging.FileHandler('testing.log')
            log.addHandler(fh)
        if correlation_id_key_val:
            log = log.bind(**correlation_id_key_val)
        return log



import json


class LambdaResponseException(Exception):
    """
    Build a custom Exception to ensure we get predictable responses back to APIG
    see: https://aws.amazon.com/blogs/compute/error-handling-patterns-in-amazon-api-gateway-and-aws-lambda/
    """

    def __init__(self, message: str, exception_type: str, error_code: str, status_code: int, request_id: str):
        # Call the base class constructor with the parameters it needs
        super(LambdaResponseException, self).__init__(message)
        self.message = message
        self.exception_type = exception_type
        self.error_code = error_code
        self.status_code = status_code
        self.request_id = request_id

    def __str__(self):
        my_error_obj = {
            'errorType': self.error_code,
            'httpStatus': self.status_code,
            'requestId': self.request_id,
            'message': self.message
        }
        return json.dumps(my_error_obj)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name='slat',
    version='0.1.9',
    author="Sam Keen",
    author_email="sam.sjk@gmail.com",
    description="Simple Lambda Toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/samkeen/SLaT",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'structlog'
    ],
)



import dataclasses
from dataclasses import dataclass
import json
from typing import Dict, Any

@dataclass()
class JsonapiBody:
    data: list
    errors: list
    meta: dict

    def __str__(self):
        return json.dumps(dataclasses.asdict(self))


JsonDict = Dict[str, Any]


from slat.types import JsonapiBody, JsonDict


class LambdaProxyResponse:

    @staticmethod
    def respond(status_code: int, resp_payload: JsonapiBody, context: object) -> JsonDict:
        """
        Returns the expected Proxy integration lambda response format
        structure mandated as per
        https://docs.aws.amazon.com/apigateway/latest/developerguide/set-up-lambda-proxy-integrations.html#api-gateway-simple-proxy-for-lambda-output-format
        :param status_code:
        :type status_code: int
        :param resp_payload:
        :type resp_payload: JsonapiBody
        :param context:
        :type context: object
        :return:
        :rtype: JsonDict
        """
        if isinstance(resp_payload, JsonapiBody):
            resp_payload.meta['request_id'] = context.aws_request_id
        response = {
            'statusCode': status_code,
            'body': '' if not resp_payload or resp_payload == {} else str(resp_payload),
            'headers': {
                'Content-Type': 'application/json',
                'X-Hb-Request-Id': context.aws_request_id,
            },
            'multiValueHeaders': {},
            'isBase64Encoded': False
        }
        return response
